#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

import datetime
import os

import pandas.compat as compat
import pandas.tseries.holiday as holiday


current_dir = os.path.dirname(__file__)
data_path = os.path.join(current_dir, 'data', 'holidays.pkl')
tse_data_path = os.path.join(current_dir, 'data', 'tseholidays.pkl')


def _read_rules(path):
    if os.path.exists(path):
        with open(path, mode='rb') as f:
            rules = compat.cPickle.load(f)
    elif __name__ != '__main__':
        raise ImportError("Unable to load '{0}'".format(path))
    else:
        rules = None
    return rules


rules = _read_rules(data_path)
tse_rules = _read_rules(tse_data_path)


class JapaneseHolidayCalendar(holiday.AbstractHolidayCalendar):
    rules = rules


class TSEHolidayCalendar(holiday.AbstractHolidayCalendar):
    rules = tse_rules


# register to pandas factory
holiday.register(JapaneseHolidayCalendar)
holiday.register(TSEHolidayCalendar)


if __name__ == '__main__':

    # https://github.com/k1LoW/holiday_jp
    import yaml

    def to_pickle(dates, path):
        rules = []
        keys = sorted(compat.iterkeys(dates))
        for key in keys:
            value = dates[key]
            dt = value['date']
            h = holiday.Holiday(value['name'], dt.year, month=dt.month, day=dt.day)
            rules.append(h)
        print(len(rules))
        with open(path, mode='wb') as w:
            compat.cPickle.dump(rules, w, protocol=2)
            print('pickled {0} data'.format(len(dates)))

    with open(os.path.join('data', 'holidays.yml'), mode='rb') as f:
        data = yaml.load(f)
    # JapaneseHolidayCalendar
    to_pickle(data, data_path)

    tse_data = data.copy()
    for y in range(1970, 2031):
        for m, d in [(1, 1), (1, 2), (1, 3), (12, 31)]:
            dt = datetime.date(y, m, d)
            if dt not in tse_data:
                tse_data[dt] = {'name': '年末年始休業日', 'date': dt}

    # TSEHolidayCalendar
    to_pickle(tse_data, tse_data_path)



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

import pandas as pd
import pandas.util.testing as tm

import japandas as jpd
from japandas.tools.tests.test_plotting import TestPlotBase, _check_plot_works


class TestDataReader(TestPlotBase):

    def test_to_yahoojp(self):
        idx = pd.DatetimeIndex(['2014-10-01', '2014-10-02', '2014-10-03'], name='日付')
        columns = jpd.io.data._ohlc_columns_jp
        expected = pd.DataFrame({'始値': [6450, 6370, 6231],
                                 '高値': [6559, 6423, 6309],
                                 '安値': [6435, 6256, 6217],
                                 '終値': [6500, 6275, 6290],
                                 '出来高': [14482100, 15240200, 10280100],
                                 '調整後終値*': [6500, 6275, 6290]},
                                index=idx, columns=columns)

        df = jpd.DataReader(7203, 'yahoojp', start='2014-10-01', end='2014-10-05')
        tm.assert_frame_equal(df, expected)
        _check_plot_works(df.plot, kind='ohlc')

        df = jpd.DataReader(7203, 'yahoojp', start='2014-10-01',
                            end='2014-10-05', interval='d')
        tm.assert_frame_equal(df, expected)

        df = jpd.DataReader(7203, 'yahoojp', start='2014-10-01',
                            end='2014-10-05', interval='v')
        tm.assert_frame_equal(df, expected)

        idx = pd.DatetimeIndex(['2014-10-06', '2014-10-14'], name='日付')
        expected = pd.DataFrame({'始値': [6370, 6050],
                                 '高値': [6455, 6069],
                                 '安値': [6145, 5710],
                                 '終値': [6220, 5731],
                                 '出来高': [51266400, 50540700],
                                 '調整後終値*': [6220, 5731]},
                                index=idx, columns=columns)

        df = jpd.DataReader(7203, 'yahoojp', start='2014-10-01',
                            end='2014-10-15', interval='w')
        tm.assert_frame_equal(df, expected)

        idx = pd.DatetimeIndex(['2014-10-01', '2014-11-01', '2014-12-01'], name='日付')
        expected = pd.DataFrame({'始値': [6450, 6830, 7360],
                                 '高値': [6559, 7314, 7873],
                                 '安値': [5710, 6696, 7107],
                                 '終値': [6498, 7314, 7558],
                                 '出来高': [233353400, 232048700, 244583800],
                                 '調整後終値*': [6498, 7314, 7558]},
                                index=idx, columns=columns)

        df = jpd.DataReader(7203, 'yahoojp', start='2014-10-01',
                            end='2014-12-31', interval='m')
        tm.assert_frame_equal(df, expected)

    def test_data_yahoojp_en(self):
        idx = pd.DatetimeIndex(['2014-10-01', '2014-10-02', '2014-10-03'], name='Date')
        columns = jpd.io.data._ohlc_columns_en
        expected = pd.DataFrame({'Open': [6450, 6370, 6231],
                                 'High': [6559, 6423, 6309],
                                 'Low': [6435, 6256, 6217],
                                 'Close': [6500, 6275, 6290],
                                 'Volume': [14482100, 15240200, 10280100],
                                 'Adj Close': [6500, 6275, 6290]},
                                index=idx, columns=columns)

        df = jpd.DataReader(7203, 'yahoojp', start='2014-10-01', end='2014-10-05', en=True)
        tm.assert_frame_equal(df, expected)
        _check_plot_works(df.plot, kind='ohlc')


if __name__ == '__main__':
    import nose
    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
                   exit=False)



#!/usr/bin/env python
# coding: utf-8

import japandas.core.strings                                                          # noqa
import japandas.io.data                                                               # noqa
from japandas.io.data import DataReader                                               # noqa
from japandas.tseries.tools import to_datetime, date_range, period_range              # noqa
from japandas.tseries.holiday import JapaneseHolidayCalendar, TSEHolidayCalendar      # noqa
import japandas.tools.plotting                                                        # noqa

from japandas.version import version as __version__                                   # noqa



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

import unittest

import pandas as pd
import pandas.compat as compat
import pandas.util.testing as tm

import japandas as jpd


class TestTools(unittest.TestCase):

    def test_to_datetime(self):
        cases = {'2011年10月30日': ('2011-10-30', '%Y-%m-%d'),
                 '2012年12月': ('2012-12', '%Y-%m'),
                 '12月3日': ('12-03', '%m-%d'),
                 '2013年9月4日10時15分': ('2013-09-04 10:15', '%Y-%m-%d %H:%M'),
                 '10月5日13時25分': ('10-05 13:25', '%m-%d %H:%M'),
                 '2014年3月8日20時45分8秒': ('2014-03-08 20:45:08', '%Y-%m-%d %H:%M:%S'),
                 '3月8日20時45分8秒': ('03-08 20:45:08', '%m-%d %H:%M:%S'),
                 '11年10月30日': ('11-10-30', '%y-%m-%d'),
                 '09年12月': ('09-12', '%y-%m'),
                 '13年9月4日10時15分': ('13-09-04 10:15', '%y-%m-%d %H:%M'),
                 '14年3月8日20時45分8秒': ('14-03-08 20:45:08', '%y-%m-%d %H:%M:%S'),
                 '14年': ('2014-01-01', '%Y-%m-%d'),
                 '2014年': ('2014-01-01', '%Y-%m-%d')
                 }

        for k, (s, f) in compat.iteritems(cases):
            result = jpd.to_datetime(k)
            expected = pd.to_datetime(s, format=f)
            self.assertEqual(result, expected)

            result = jpd.to_datetime([k])
            expected = pd.to_datetime([s], format=f)
            tm.assert_index_equal(result, expected)

            result = jpd.to_datetime([k], box=False)
            expected = pd.to_datetime([s], format=f, box=False)
            tm.assert_numpy_array_equal(result, expected)

    def test_date_range(self):
        result = jpd.date_range(start='2013年11月15日', end='2014年12月18日', freq='D')
        expected = pd.date_range(start='2013-11-15', end='2014-12-18', freq='D')
        tm.assert_index_equal(result, expected)

        result = jpd.date_range(start='2014年1月15日09時25分', end='2014年1月18日10時45分', freq='M')
        expected = pd.date_range(start='2014-01-15 09:25', end='2014-01-18 10:45', freq='M')
        tm.assert_index_equal(result, expected)

    def test_period_range(self):
        result = jpd.period_range(start='2013年11月15日', end='2014年12月18日', freq='D')
        expected = pd.period_range(start='2013-11-15', end='2014-12-18', freq='D')
        tm.assert_index_equal(result, expected)

        result = jpd.period_range(start='2014年1月15日09時25分', end='2014年1月18日10時45分', freq='M')
        expected = pd.period_range(start='2014-01-15 09:25', end='2014-01-18 10:45', freq='M')
        tm.assert_index_equal(result, expected)


if __name__ == '__main__':
    import nose
    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
                   exit=False)



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

import time

import pandas as pd
from japandas.io.estat import EStatReader

from pandas_datareader import data
from pandas_datareader.yahoo.daily import YahooDailyReader


_ohlc_columns_jp = ['始値', '高値', '安値', '終値', '出来高', '調整後終値*']
_ohlc_columns_en = ['Open', 'High', 'Low', 'Close', 'Volume', 'Adj Close']


_SLEEP_TIME = 0.5


class YahooJPReader(YahooDailyReader):

    def __init__(self, en=False, **kwargs):
        super(YahooJPReader, self).__init__(**kwargs)
        self.en = en

    @property
    def url(self):
        return 'http://info.finance.yahoo.co.jp/history/'

    def _get_params(self, symbol):
        params = {
            'code': symbol,
            'sm': self.start.month,
            'sd': self.start.day,
            'sy': self.start.year,
            'em': self.end.month,
            'ed': self.end.day,
            'ey': self.end.year,
            'tm': self.interval,
            'p': 1
        }
        return params

    def read(self):
        # Use _DailyBaseReader's definition
        df = self._read_one_data(self.url, params=self._get_params(self.symbols))
        return df

    def _read_one_data(self, url, params):
        base = (url + '?code={code}.T&sy={sy}&sm={sm}&sd={sd}&'
                'ey={ey}&em={em}&ed={ed}&tm={tm}&p={p}')
        results = []
        while True:
            url = base.format(**params)
            tables = pd.read_html(url, header=0)
            if len(tables) < 2 or len(tables[1]) == 0:
                break
            results.append(tables[1])
            params['p'] = params['p'] + 1
            time.sleep(_SLEEP_TIME)
        result = pd.concat(results, ignore_index=True)

        if self.en:
            result.columns = ['Date'] + _ohlc_columns_en
            dtkey = 'Date'
        else:
            dtkey = '日付'

        if self.interval == 'm':
            result[dtkey] = pd.to_datetime(result[dtkey], format='%Y年%m月')
        else:
            result[dtkey] = pd.to_datetime(result[dtkey], format='%Y年%m月%d日')
        result = result.set_index(dtkey)
        result = result.sort_index()
        return result


def DataReader(symbols, data_source=None, start=None, end=None, appid=None, **kwargs):
    if data_source == 'yahoojp':
        return YahooJPReader(symbols=symbols, start=start,
                             end=end, **kwargs).read()
    elif data_source == 'estat':
        return EStatReader(symbols=symbols, appid=appid, **kwargs).read()
    else:
        return data.DataReader(name=symbols, data_source=data_source,
                               start=start, end=end, **kwargs)


DataReader.__doc__ = data.DataReader.__doc__


if __name__ == '__main__':
    toyota_tse = DataReader(7203, 'yahoojp', start='2014-10-01')
    print(toyota_tse.head())



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

import pandas as pd
try:
    import pandas.plotting._core as plotting
except ImportError:
    import pandas.tools.plotting as plotting

from japandas.compat import PANDAS_0180, PANDAS_0200
from japandas.io.data import _ohlc_columns_jp, _ohlc_columns_en


class OhlcPlot(plotting.LinePlot):
    ohlc_cols = pd.Index(['open', 'high', 'low', 'close'])
    reader_cols_en = pd.Index(_ohlc_columns_en)
    reader_cols_jp = pd.Index(_ohlc_columns_jp)

    def __init__(self, data, **kwargs):
        data = data.copy()
        self.freq = kwargs.pop('freq', 'B')

        if isinstance(data, pd.Series):
            if PANDAS_0180:
                data = data.resample(self.freq).ohlc()
            else:
                data = data.resample(self.freq, how='ohlc')
        assert isinstance(data, pd.DataFrame)
        assert isinstance(data.index, pd.DatetimeIndex)

        if data.columns.equals(self.ohlc_cols):
            data.columns = [c.title() for c in data.columns]
        elif data.columns.equals(self.reader_cols_jp):
            data.columns = self.reader_cols_en
        elif data.columns.equals(self.reader_cols_en):
            pass
        else:

            raise ValueError('data is not ohlc-like:')
        data = data[['Open', 'Close', 'High', 'Low']]
        plotting.LinePlot.__init__(self, data, **kwargs)

    def _get_plot_function(self):
        try:
            from matplotlib.finance import candlestick
        except ImportError:
            from matplotlib.finance import candlestick_ohlc as candlestick

        def _plot(data, ax, **kwds):
            candles = candlestick(ax, data.values, **kwds)
            return candles

        return _plot

    def _make_plot(self):
        try:
            from pandas.plotting._timeseries import (_decorate_axes,
                                                     format_dateaxis)
        except ImportError:
            from pandas.tseries.plotting import _decorate_axes, format_dateaxis
        plotf = self._get_plot_function()
        ax = self._get_ax(0)

        data = self.data
        data.index.name = 'Date'
        data = data.to_period(freq=self.freq)
        index = data.index
        data = data.reset_index(level=0)

        if self._is_ts_plot():
            data['Date'] = data['Date'].apply(lambda x: x.ordinal)
            _decorate_axes(ax, self.freq, self.kwds)
            candles = plotf(data, ax, **self.kwds)
            if PANDAS_0200:
                format_dateaxis(ax, self.freq, index)
            else:
                format_dateaxis(ax, self.freq)
        else:
            from matplotlib.dates import date2num, AutoDateFormatter, AutoDateLocator

            data['Date'] = data['Date'].apply(lambda x: date2num(x.to_timestamp()))
            candles = plotf(data, ax, **self.kwds)

            locator = AutoDateLocator()
            ax.xaxis.set_major_locator(locator)
            ax.xaxis.set_major_formatter(AutoDateFormatter(locator))

        return candles


if 'ohlc' not in plotting._plot_klass:
    plotting._all_kinds.append('ohlc')
    plotting._common_kinds.append('ohlc')
    plotting._plot_klass['ohlc'] = OhlcPlot



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

import datetime
import unittest

import pandas as pd

import japandas as jpd


class TestCalendar(unittest.TestCase):

    def setUp(self):
        self.expected = [datetime.datetime(2014, 1, 1, 0, 0),
                         datetime.datetime(2014, 1, 13, 0, 0),
                         datetime.datetime(2014, 2, 11, 0, 0),
                         datetime.datetime(2014, 3, 21, 0, 0),
                         datetime.datetime(2014, 4, 29, 0, 0),
                         datetime.datetime(2014, 5, 3, 0, 0),
                         datetime.datetime(2014, 5, 4, 0, 0),
                         datetime.datetime(2014, 5, 5, 0, 0),
                         datetime.datetime(2014, 5, 6, 0, 0),
                         datetime.datetime(2014, 7, 21, 0, 0),
                         datetime.datetime(2014, 9, 15, 0, 0),
                         datetime.datetime(2014, 9, 23, 0, 0),
                         datetime.datetime(2014, 10, 13, 0, 0),
                         datetime.datetime(2014, 11, 3, 0, 0),
                         datetime.datetime(2014, 11, 23, 0, 0),
                         datetime.datetime(2014, 11, 24, 0, 0),
                         datetime.datetime(2014, 12, 23, 0, 0)]

        self.start_date = datetime.datetime(2014, 1, 1)
        self.end_date = datetime.datetime(2014, 12, 31)

    def test_calendar(self):

        calendar = jpd.JapaneseHolidayCalendar()
        holidays_0 = calendar.holidays(self.start_date,
                                       self.end_date)

        holidays_1 = calendar.holidays(self.start_date.strftime('%Y-%m-%d'),
                                       self.end_date.strftime('%Y-%m-%d'))
        holidays_2 = calendar.holidays(pd.Timestamp(self.start_date),
                                       pd.Timestamp(self.end_date))

        self.assertEqual(holidays_0.to_pydatetime().tolist(), self.expected)
        self.assertEqual(holidays_1.to_pydatetime().tolist(), self.expected)
        self.assertEqual(holidays_2.to_pydatetime().tolist(), self.expected)

    def test_cday(self):
        calendar = jpd.JapaneseHolidayCalendar()
        cday = pd.offsets.CDay(calendar=calendar)

        dt = datetime.datetime(2014, 1, 12)
        self.assertEqual(dt - cday, datetime.datetime(2014, 1, 10))
        self.assertEqual(dt + cday, datetime.datetime(2014, 1, 14))

        dt = datetime.datetime(2014, 1, 10)
        self.assertEqual(dt - cday, datetime.datetime(2014, 1, 9))
        self.assertEqual(dt + cday, datetime.datetime(2014, 1, 14))

        dt = datetime.datetime(2014, 4, 28)
        self.assertEqual(dt - cday, datetime.datetime(2014, 4, 25))
        self.assertEqual(dt + cday, datetime.datetime(2014, 4, 30))

        dt = datetime.datetime(2014, 5, 3)
        self.assertEqual(dt - cday, datetime.datetime(2014, 5, 2))
        self.assertEqual(dt + cday, datetime.datetime(2014, 5, 7))

        dt = datetime.datetime(2014, 5, 6)
        self.assertEqual(dt - cday, datetime.datetime(2014, 5, 2))
        self.assertEqual(dt + cday, datetime.datetime(2014, 5, 7))

    def test_factory(self):
        calendar = pd.tseries.holiday.get_calendar('JapaneseHolidayCalendar')
        self.assertTrue(isinstance(calendar, jpd.JapaneseHolidayCalendar))

        calendar = pd.tseries.holiday.get_calendar('TSEHolidayCalendar')
        self.assertTrue(isinstance(calendar, jpd.TSEHolidayCalendar))

    def test_holiday_attributes(self):
        calendar = jpd.JapaneseHolidayCalendar()
        self.assertEqual(calendar.rules[0].name, '元日')
        self.assertEqual(calendar.rules[0].year, 1970)
        self.assertEqual(calendar.rules[0].month, 1)
        self.assertEqual(calendar.rules[0].day, 1)

    def test_jpholiday_holidays(self):
        calendar = jpd.JapaneseHolidayCalendar()
        holidays = calendar.holidays()
        for y in range(1970, 2030):
            for m, d in [(1, 1)]:
                dt = datetime.date(y, m, d)
                self.assertTrue(dt in holidays)

        for e in self.expected:
            self.assertTrue(dt in holidays)

    def test_tseholiday_holidays(self):
        calendar = jpd.TSEHolidayCalendar()
        holidays = calendar.holidays()
        for y in range(1970, 2031):
            for m, d in [(1, 1), (1, 2), (1, 3), (12, 31)]:
                dt = datetime.date(y, m, d)
                self.assertTrue(dt in holidays)

        # test initial / final date explicitly
        self.assertTrue(datetime.date(1970, 1, 1) in holidays)
        self.assertTrue(datetime.date(2030, 12, 31) in holidays)
        for e in self.expected:
            self.assertTrue(dt in holidays)

    def test_holiday_bug(self):
        # GH 42

        for calendar in [jpd.TSEHolidayCalendar(),
                         jpd.JapaneseHolidayCalendar()]:
            holidays = calendar.holidays()

            self.assertFalse(datetime.datetime(1993, 9, 5) in holidays)
            self.assertTrue(datetime.datetime(1993, 9, 15) in holidays)

            self.assertFalse(datetime.datetime(2020, 8, 12) in holidays)
            self.assertTrue(datetime.datetime(2020, 8, 11) in holidays)


if __name__ == '__main__':
    import nose
    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
                   exit=False)



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

from unicodedata import normalize

from pandas.compat import PY3, iteritems, u_safe
import pandas.core.strings as strings


# soundmarks require special handlings
_HKANA = 'ｧｱｨｲｩｳｪｴｫｵｶｷｸｹｺｻｼｽｾｿﾀﾁｯﾂﾃﾄﾅﾆﾇﾈﾉﾊﾋﾌﾍﾎﾏﾐﾑﾒﾓｬﾔｭﾕｮﾖﾗﾘﾙﾚﾛﾜｦﾝﾞｰ･｢｣｡､'
_ZALPHA = ('ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ'
           'ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ')
_ZSYMBOL = '！＂＃＄％＆＇（）＊＋，－．／：；＜＝＞？＠［＼］＾＿｀｛｜｝～　'
_ZDIGIT = '０１２３４５６７８９'

# mapping from full-width to half-width
_KANA_MAPPER = {normalize('NFKC', c): c for c in _HKANA}
_ALPHA_MAPPER = {c: normalize('NFKC', c) for c in _ZALPHA}
_DIGIT_MAPPER = {c: normalize('NFKC', c) for c in _ZDIGIT}
_SYMBOL_MAPPER = {c: normalize('NFKC', c) for c in _ZSYMBOL}


def _reverse_dict(dict):
    return {v: k for k, v in iteritems(dict)}


def _ord_dict(dict):
    return {ord(k): v for k, v in iteritems(dict)}


# for unicode.translate
_Z2H_KANA = _ord_dict(_KANA_MAPPER)
_Z2H_ALPHA = _ord_dict(_ALPHA_MAPPER)
_Z2H_DIGIT = _ord_dict(_DIGIT_MAPPER)
_Z2H_SYMBOL = _ord_dict(_SYMBOL_MAPPER)
_H2Z_KANA = _ord_dict(_reverse_dict(_KANA_MAPPER))
_H2Z_ALPHA = _ord_dict(_reverse_dict(_ALPHA_MAPPER))
_H2Z_DIGIT = _ord_dict(_reverse_dict(_DIGIT_MAPPER))
_H2Z_SYMBOL = _ord_dict(_reverse_dict(_SYMBOL_MAPPER))


def _h2z_sm(text):
    return (text.replace("ｶﾞ", "ガ").replace("ｷﾞ", "ギ").replace("ｸﾞ", "グ").replace("ｹﾞ", "ゲ").
            replace("ｺﾞ", "ゴ").replace("ｻﾞ", "ザ").replace("ｼﾞ", "ジ").replace("ｽﾞ", "ズ").
            replace("ｾﾞ", "ゼ").replace("ｿﾞ", "ゾ").replace("ﾀﾞ", "ダ").replace("ﾁﾞ", "ヂ").
            replace("ﾂﾞ", "ヅ").replace("ﾃﾞ", "デ").replace("ﾄﾞ", "ド").replace("ﾊﾞ", "バ").
            replace("ﾋﾞ", "ビ").replace("ﾌﾞ", "ブ").replace("ﾍﾞ", "ベ").replace("ﾎﾞ", "ボ").
            replace("ﾊﾟ", "パ").replace("ﾋﾟ", "ピ").replace("ﾌﾟ", "プ").replace("ﾍﾟ", "ペ").
            replace("ﾎﾟ", "ポ").replace("ｳﾞ", "ヴ"))


def _z2h_sm(text):
    return (text.replace("ガ", "ｶﾞ").replace("ギ", "ｷﾞ").replace("グ", "ｸﾞ").replace("ゲ", "ｹﾞ").
            replace("ゴ", "ｺﾞ").replace("ザ", "ｻﾞ").replace("ジ", "ｼﾞ").replace("ズ", "ｽﾞ").
            replace("ゼ", "ｾﾞ").replace("ゾ", "ｿﾞ").replace("ダ", "ﾀﾞ").replace("ヂ", "ﾁﾞ").
            replace("ヅ", "ﾂﾞ").replace("デ", "ﾃﾞ").replace("ド", "ﾄﾞ").replace("バ", "ﾊﾞ").
            replace("ビ", "ﾋﾞ").replace("ブ", "ﾌﾞ").replace("ベ", "ﾍﾞ").replace("ボ", "ﾎﾞ").
            replace("パ", "ﾊﾟ").replace("ピ", "ﾋﾟ").replace("プ", "ﾌﾟ").replace("ペ", "ﾍﾟ").
            replace("ポ", "ﾎﾟ").replace("ヴ", "ｳﾞ"))


def str_z2h(self, kana=True, alpha=True, digit=True, symbol=True):
    mapper = dict()
    if kana:
        mapper.update(_Z2H_KANA)
    if alpha:
        mapper.update(_Z2H_ALPHA)
    if digit:
        mapper.update(_Z2H_DIGIT)
    if symbol:
        mapper.update(_Z2H_SYMBOL)

    if kana:
        if PY3:
            def f(x):
                return _z2h_sm(x).translate(mapper)
        else:
            def f(x):
                return _z2h_sm(u_safe(x)).translate(mapper)
    else:
        if PY3:
            def f(x):
                return x.translate(mapper)
        else:
            def f(x):
                return u_safe(x).translate(mapper)

    try:
        target = self.series
    except AttributeError:
        target = self._data
    return self._wrap_result(strings._na_map(f, target))


def str_h2z(self, kana=True, alpha=True, digit=True, symbol=True):
    mapper = dict()
    if kana:
        mapper.update(_H2Z_KANA)
    if alpha:
        mapper.update(_H2Z_ALPHA)
    if digit:
        mapper.update(_H2Z_DIGIT)
    if symbol:
        mapper.update(_H2Z_SYMBOL)

    if kana:
        if PY3:
            def f(x):
                return _h2z_sm(x).translate(mapper)
        else:
            def f(x):
                return _h2z_sm(u_safe(x)).translate(mapper)
    else:
        if PY3:
            def f(x):
                return x.translate(mapper)
        else:
            def f(x):
                return u_safe(x).translate(mapper)

    try:
        target = self.series
    except AttributeError:
        target = self._data
    return self._wrap_result(strings._na_map(f, target))


# do not overwrite existing func
if not hasattr(strings.StringMethods, 'z2h'):
    strings.StringMethods.z2h = str_z2h


if not hasattr(strings.StringMethods, 'h2z'):
    strings.StringMethods.h2z = str_h2z



#!/usr/bin/env python
# coding: utf-8

import pandas as pd
from distutils.version import LooseVersion


PANDAS_VERSION = LooseVersion(pd.__version__)

if PANDAS_VERSION >= LooseVersion('0.20.0'):
    PANDAS_0200 = True
else:
    PANDAS_0200 = False

if PANDAS_VERSION >= LooseVersion('0.19.0'):
    PANDAS_0190 = True
else:
    PANDAS_0190 = False

if PANDAS_VERSION >= LooseVersion('0.18.0'):
    PANDAS_0180 = True
else:
    PANDAS_0180 = False



#!/usr/bin/env python
# coding: utf-8

# do not import unicode_literals here to test ASCII in Python 2.7
import unittest

import pandas as pd
import pandas.util.testing as tm


class TestStrings(unittest.TestCase):

    def setUp(self):
        self.zhiragana_s = pd.Series([u'ぁあぃいぅうぇえぉお',
                                      u'かがきぎくぐけげこご',
                                      u'さざしじすずせぜそぞ',
                                      u'ただちぢっつづてでとど',
                                      u'なにぬねの',
                                      u'はばぱひびぴふぶぷへべぺほぼぽ',
                                      u'まみむめもゃやゅゆょよ',
                                      u'らりるれろわをんゎゐゑゕゖゔ'])

        self.zkatakana_s = pd.Series([u'ァアィイゥウェエォオ',
                                      u'カガキギクグケゲコゴ',
                                      u'サザシジスズセゼソゾ',
                                      u'タダチヂッツヅテデトド',
                                      u'ナニヌネノ',
                                      u'ハバパヒビピフブプヘベペホボポ',
                                      u'マミムメモャヤュユョヨ',
                                      u'ラリルレロワヲンヮヰヱヵヶヴ',
                                      u'ー・「」。、'])

        self.hkatakana_s = pd.Series([u'ｧｱｨｲｩｳｪｴｫｵ',
                                      u'ｶｶﾞｷｷﾞｸｸﾞｹｹﾞｺｺﾞ',
                                      u'ｻｻﾞｼｼﾞｽｽﾞｾｾﾞｿｿﾞ',
                                      u'ﾀﾀﾞﾁﾁﾞｯﾂﾂﾞﾃﾃﾞﾄﾄﾞ',
                                      u'ﾅﾆﾇﾈﾉ',
                                      u'ﾊﾊﾞﾊﾟﾋﾋﾞﾋﾟﾌﾌﾞﾌﾟﾍﾍﾞﾍﾟﾎﾎﾞﾎﾟ',
                                      u'ﾏﾐﾑﾒﾓｬﾔｭﾕｮﾖ',
                                      u'ﾗﾘﾙﾚﾛﾜｦﾝヮヰヱヵヶｳﾞ',
                                      u'ｰ･｢｣｡､'])

        self.zalpha_s = pd.Series([u'ＡＢＣＤＥＦＧＨ',
                                   u'ＩＪＫＬＭＮＯＰ',
                                   u'ＱＲＳＴＵＶＷＸＹＺ',
                                   u'ａｂｃｄｅｆｇｈ',
                                   u'ｉｊｋｌｍｎｏｐ',
                                   u'ｑｒｓｔｕｖｗｘｙｚ'])
        self.halpha_s = pd.Series(['ABCDEFGH',
                                   'IJKLMNOP',
                                   'QRSTUVWXYZ',
                                   'abcdefgh',
                                   'ijklmnop',
                                   'qrstuvwxyz'])

        self.zdigit_s = pd.Series([u'０１２３４', u'５６７８９'])
        self.hdigit_s = pd.Series(['01234', '56789'])

        self.zsymbol_s = pd.Series([u'！＂＃＄％＆',
                                    u'＇（）＊＋，',
                                    u'－．／：；＜',
                                    u'＝＞？＠［＼',
                                    u'］＾＿｀～｛',
                                    u'｜｝　'])
        self.hsymbol_s = pd.Series([u'!"#$%&',
                                    u"'()*+,",
                                    u'-./:;<',
                                    u'=>?@[\\',
                                    u']^_`~{',
                                    u'|} '])

    def test_mapper(self):
        import japandas.core.strings as s
        self.assertEqual(len(s._KANA_MAPPER), len(s._HKANA))
        self.assertEqual(len(s._ALPHA_MAPPER), len(s._ZALPHA))
        self.assertEqual(len(s._DIGIT_MAPPER), len(s._ZDIGIT))
        self.assertEqual(len(s._SYMBOL_MAPPER), len(s._ZSYMBOL))

        self.assertEqual(len(s._reverse_dict(s._KANA_MAPPER)), len(s._HKANA))
        self.assertEqual(len(s._reverse_dict(s._ALPHA_MAPPER)), len(s._ZALPHA))
        self.assertEqual(len(s._reverse_dict(s._DIGIT_MAPPER)), len(s._ZDIGIT))
        self.assertEqual(len(s._Z2H_SYMBOL), len(s._H2Z_SYMBOL))

        self.assertEqual(len(s._Z2H_KANA), len(s._H2Z_KANA))
        self.assertEqual(len(s._Z2H_ALPHA), len(s._H2Z_ALPHA))
        self.assertEqual(len(s._Z2H_DIGIT), len(s._H2Z_DIGIT))
        self.assertEqual(len(s._reverse_dict(s._SYMBOL_MAPPER)), len(s._ZSYMBOL))

    def test_z2h(self):
        s = pd.Series([u'ａａａ', 'bbb', u'アアア', u'１', u'＊'])
        result = s.str.z2h()
        expected = pd.Series(['aaa', 'bbb', u'ｱｱｱ', '1', '*'])
        tm.assert_series_equal(result, expected)

        # full-width kana to half-width kana
        result = self.zkatakana_s.str.z2h(kana=True, alpha=False, digit=False, symbol=False)
        tm.assert_series_equal(result, self.hkatakana_s)
        result = self.zkatakana_s.str.z2h(kana=False, alpha=True, digit=False, symbol=False)
        tm.assert_series_equal(result, self.zkatakana_s)
        result = self.zkatakana_s.str.z2h(kana=False, alpha=False, digit=True, symbol=False)
        tm.assert_series_equal(result, self.zkatakana_s)
        result = self.zkatakana_s.str.z2h(kana=False, alpha=False, digit=False, symbol=True)
        tm.assert_series_equal(result, self.zkatakana_s)

        # full-width kana to half-width alpha
        result = self.zalpha_s.str.z2h(kana=True, alpha=False, digit=False, symbol=False)
        tm.assert_series_equal(result, self.zalpha_s)
        result = self.zalpha_s.str.z2h(kana=False, alpha=True, digit=False, symbol=False)
        tm.assert_series_equal(result, self.halpha_s)
        result = self.zalpha_s.str.z2h(kana=False, alpha=False, digit=True, symbol=False)
        tm.assert_series_equal(result, self.zalpha_s)
        result = self.zalpha_s.str.z2h(kana=False, alpha=False, digit=False, symbol=True)
        tm.assert_series_equal(result, self.zalpha_s)

        # full-width kana to half-width digit
        result = self.zdigit_s.str.z2h(kana=True, alpha=False, digit=False, symbol=False)
        tm.assert_series_equal(result, self.zdigit_s)
        result = self.zdigit_s.str.z2h(kana=False, alpha=True, digit=False, symbol=False)
        tm.assert_series_equal(result, self.zdigit_s)
        result = self.zdigit_s.str.z2h(kana=False, alpha=False, digit=True, symbol=False)
        tm.assert_series_equal(result, self.hdigit_s)
        result = self.zdigit_s.str.z2h(kana=False, alpha=False, digit=False, symbol=True)
        tm.assert_series_equal(result, self.zdigit_s)

        # full-width kana to half-width symbol
        result = self.zsymbol_s.str.z2h(kana=True, alpha=False, digit=False, symbol=False)
        tm.assert_series_equal(result, self.zsymbol_s)
        result = self.zsymbol_s.str.z2h(kana=False, alpha=True, digit=False, symbol=False)
        tm.assert_series_equal(result, self.zsymbol_s)
        result = self.zsymbol_s.str.z2h(kana=False, alpha=False, digit=True, symbol=False)
        tm.assert_series_equal(result, self.zsymbol_s)
        result = self.zsymbol_s.str.z2h(kana=False, alpha=False, digit=False, symbol=True)
        tm.assert_series_equal(result, self.hsymbol_s)

        # half-width to half-width
        result = self.hkatakana_s.str.z2h()
        tm.assert_series_equal(result, self.hkatakana_s)
        result = self.halpha_s.str.z2h()
        tm.assert_series_equal(result, self.halpha_s)
        result = self.hdigit_s.str.z2h()
        tm.assert_series_equal(result, self.hdigit_s)
        result = self.hsymbol_s.str.z2h()
        tm.assert_series_equal(result, self.hsymbol_s)

    def test_h2z(self):
        s = pd.Series(['aaa', 'bbb', u'ｱｱｱ', u'１', '*'])
        result = s.str.h2z()
        expected = pd.Series([u'ａａａ', u'ｂｂｂ', u'アアア', u'１', u'＊'])
        tm.assert_series_equal(result, expected)

        # half-width kana to full-width kana
        result = self.hkatakana_s.str.h2z(kana=True, alpha=False, digit=False, symbol=False)
        tm.assert_series_equal(result, self.zkatakana_s)
        result = self.hkatakana_s.str.h2z(kana=False, alpha=True, digit=False, symbol=False)
        tm.assert_series_equal(result, self.hkatakana_s)
        result = self.hkatakana_s.str.h2z(kana=False, alpha=False, digit=True, symbol=False)
        tm.assert_series_equal(result, self.hkatakana_s)
        result = self.hkatakana_s.str.h2z(kana=False, alpha=False, digit=False, symbol=True)
        tm.assert_series_equal(result, self.hkatakana_s)

        # half-width kana to full-width alpha
        result = self.halpha_s.str.h2z(kana=True, alpha=False, digit=False, symbol=False)
        tm.assert_series_equal(result, self.halpha_s)
        result = self.halpha_s.str.h2z(kana=False, alpha=True, digit=False, symbol=False)
        tm.assert_series_equal(result, self.zalpha_s)
        result = self.halpha_s.str.h2z(kana=False, alpha=False, digit=True, symbol=False)
        tm.assert_series_equal(result, self.halpha_s)
        result = self.halpha_s.str.h2z(kana=False, alpha=False, digit=False, symbol=True)
        tm.assert_series_equal(result, self.halpha_s)

        # half-width kana to full-width digit
        result = self.hdigit_s.str.h2z(kana=True, alpha=False, digit=False, symbol=False)
        tm.assert_series_equal(result, self.hdigit_s)
        result = self.hdigit_s.str.h2z(kana=False, alpha=True, digit=False, symbol=False)
        tm.assert_series_equal(result, self.hdigit_s)
        result = self.hdigit_s.str.h2z(kana=False, alpha=False, digit=True, symbol=False)
        tm.assert_series_equal(result, self.zdigit_s)
        result = self.hdigit_s.str.h2z(kana=False, alpha=False, digit=False, symbol=True)
        tm.assert_series_equal(result, self.hdigit_s)

        # half-width kana to full-width symbol
        result = self.hsymbol_s.str.h2z(kana=True, alpha=False, digit=False, symbol=False)
        tm.assert_series_equal(result, self.hsymbol_s)
        result = self.hsymbol_s.str.h2z(kana=False, alpha=True, digit=False, symbol=False)
        tm.assert_series_equal(result, self.hsymbol_s)
        result = self.hsymbol_s.str.h2z(kana=False, alpha=False, digit=True, symbol=False)
        tm.assert_series_equal(result, self.hsymbol_s)
        result = self.hsymbol_s.str.h2z(kana=False, alpha=False, digit=False, symbol=True)
        tm.assert_series_equal(result, self.zsymbol_s)

        # full-width to full-width
        result = self.zkatakana_s.str.h2z()
        tm.assert_series_equal(result, self.zkatakana_s)
        result = self.zalpha_s.str.h2z()
        tm.assert_series_equal(result, self.zalpha_s)
        result = self.zdigit_s.str.h2z()
        tm.assert_series_equal(result, self.zdigit_s)
        result = self.zkatakana_s.str.h2z()
        tm.assert_series_equal(result, self.zkatakana_s)

    def test_z2h_obj(self):
        s = pd.Series(['aaa', None, u'アアア', u'あああ', u'１', 3])
        result = s.str.z2h()
        expected = pd.Series(['aaa', None, u'ｱｱｱ', u'あああ', '1', None])
        tm.assert_series_equal(result, expected)

        empty_str = pd.Series(dtype=str)
        tm.assert_series_equal(empty_str.str.h2z(), empty_str)

    def test_h2z_obj(self):
        s = pd.Series(['aaa', None, u'ｱｱｱ', u'あああ', u'１', 3])
        result = s.str.h2z()
        expected = pd.Series([u'ａａａ', None, u'アアア', u'あああ', u'１', None])
        tm.assert_series_equal(result, expected)

        empty_str = pd.Series(dtype=str)
        tm.assert_series_equal(empty_str.str.h2z(), empty_str)

    def test_normalize(self):
        s = pd.Series([u'ａａａ', 'bbb', u'ｱｱｱ', u'１', u'＊'])
        result = s.str.normalize('NFKC')
        expected = pd.Series(['aaa', 'bbb', u'アアア', '1', '*'])
        tm.assert_series_equal(result, expected)

        s = pd.Series([u'ａａａ', None, 'bbb', u'ｱｱｱ', u'１', 5, u'＊'])
        result = s.str.normalize('NFKC')
        expected = pd.Series(['aaa', None, 'bbb', u'アアア', '1', None, '*'])
        tm.assert_series_equal(result, expected)

        empty_str = pd.Series(dtype=str)
        tm.assert_series_equal(empty_str.str.normalize('NFKC'), empty_str)

    def test_normalize_format(self):
        import unicodedata
        values = [u'ｱｲｳｴｵ', u'ｶｷｸｹｺ', u'ｶﾞｷﾞｸﾞｹﾞｺﾞ', u'ＡＢＣＤＥ']
        for format in ['NFD', 'NFC', 'NFKD', 'NFKC']:
            result = pd.Series(values).str.normalize(format).tolist()
            expected = [unicodedata.normalize(format, v) for v in values]
            self.assertEqual(result, expected)


if __name__ == '__main__':
    import nose
    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
                   exit=False)



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

import numpy as np
import pandas as pd

from japandas.compat import PANDAS_0190, PANDAS_0180


if PANDAS_0190:
    from pandas.tests.plotting.common import TestPlotBase, _check_plot_works
else:
    from pandas.tests.test_graphics import TestPlotBase, _check_plot_works


class TestTools(TestPlotBase):

    def test_to_ohlc(self):
        n = 50
        idx = pd.date_range(start='2014-10-01 09:00', freq='H', periods=n)
        s = pd.Series(np.random.randn(n), index=idx)
        _check_plot_works(s.plot, kind='ohlc')
        _check_plot_works(s.plot, kind='ohlc', x_compat=True)

        if PANDAS_0180:
            ohlc = s.resample('B').ohlc()
        else:
            ohlc = s.resample('B', how='ohlc')
        _check_plot_works(ohlc.plot, kind='ohlc')
        _check_plot_works(ohlc.plot, kind='ohlc', x_compat=True)


if __name__ == '__main__':
    import nose
    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
                   exit=False)



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

import os
import unittest

import numpy as np
import pandas as pd
import pandas.util.testing as tm
import japandas as jpd


class TestEstat(unittest.TestCase):

    def test_data_estat_error(self):
        with self.assertRaises(ValueError):
            # no app ID
            jpd.DataReader('00200521', 'estat', appid=None)

        ESTAT_KEY = os.environ['ESTAT']

        with self.assertRaises(ValueError):
            # blank list
            jpd.DataReader([], 'estat', appid=ESTAT_KEY)

        with self.assertRaises(ValueError):
            # invalid type
            jpd.DataReader(1, 'estat', appid=ESTAT_KEY)

    def test_data_estat_list(self):

        ESTAT_KEY = os.environ['ESTAT']
        df = jpd.DataReader('00200521', 'estat', appid=ESTAT_KEY)

        exp_columns = pd.Index(['統計表ID', '政府統計名',
                                '作成機関名', '提供統計名及び提供分類名',
                                '統計表題名及び表番号', '提供周期', '調査年月',
                                '公開日', '小地域属性フラグ', '統計大分野名',
                                '統計小分野名', '総件数', '最終更新日'],)
        tm.assert_index_equal(df.columns, exp_columns)

        target = df.head(n=3)
        df = jpd.DataReader(target, 'estat', appid=ESTAT_KEY)
        self.assertIsInstance(df, pd.DataFrame)

        df = jpd.DataReader('00200523', 'estat', appid=ESTAT_KEY)
        tm.assert_index_equal(df.columns, exp_columns)

    def test_data_estat_list_all(self):
        # 以下 すべての提供データをテスト
        # http://www.e-stat.go.jp/api/api-data/

        targets = ['00200521', '00200522', '00200523', '00200524', '00200531',
                   '00200532', '00200533', '00200541', '00200543', '00200544',
                   '00200545', '00200551', '00200552', '00200553', '00200561',
                   '00200563', '00200564', '00200565', '00200566', '00200571',
                   '00200572', '00200573',    # '00200511', '00200502', (no data found)
                   '00250011']
        for target in targets:
            self._assert_target(target)

    def test_data_estat_list_all2(self):
        # Travis CI でのタイムアウトを防ぐため分割
        targets = ['00350600', '00350620', '00351000', '00400001', '00400002',
                   '00400003', '00400004', '00400202', '00450011', '00450012',
                   '00450021', '00450022', '00450061', '00450071', '00450091',
                   '00450151', '00500201', '00500209', '00500215', '00500216',
                   '00500217', '00500225', '00550010', '00550020', '00550030',
                   '00550040', '00550100', '00550200', '00550210', '00551020',
                   '00551130', '00600330', '00600470', '00600480']
        for target in targets:
            self._assert_target(target)

    def _assert_target(self, target):
        ESTAT_KEY = os.environ['ESTAT']

        df = jpd.DataReader(target, 'estat', appid=ESTAT_KEY)
        exp_columns = pd.Index(['統計表ID', '政府統計名',
                                '作成機関名', '提供統計名及び提供分類名',
                                '統計表題名及び表番号', '提供周期', '調査年月',
                                '公開日', '小地域属性フラグ', '統計大分野名',
                                '統計小分野名', '総件数', '最終更新日'],)
        tm.assert_index_equal(df.columns, exp_columns)

        target = df.head(n=3)
        df = jpd.DataReader(target, 'estat', appid=ESTAT_KEY)
        self.assertIsInstance(df, pd.DataFrame)

    def test_data_estat_data(self):

        ESTAT_KEY = os.environ['ESTAT']
        df = jpd.DataReader('0000030001', 'estat', appid=ESTAT_KEY)

        exp = pd.DataFrame({'value': [117060396, 89187409, 27872987, 5575989, 1523907],
                            '全国都道府県030001': ['全国', '全国市部', '全国郡部', '北海道', '青森県'],
                            '全域・集中の別030002': ['全域'] * 5,
                            '年齢５歳階級Ａ030002': ['総数'] * 5,
                            '男女Ａ030001': ['男女総数'] * 5},
                           index=pd.DatetimeIndex(['1980-01-01'] * 5, name='時間軸(年次)'))
        tm.assert_frame_equal(df.head(), exp)

        df = jpd.DataReader(['0000030001', '0000030002'], 'estat', appid=ESTAT_KEY)
        self.assertIsInstance(df, pd.DataFrame)

        df = jpd.DataReader("0002180001", 'estat', appid=ESTAT_KEY)
        exp = pd.DataFrame({'value': [445007, 194243, 199623, 203464, 190711],
                            '全国・都道府県・大都市': ['全国'] * 5,
                            '性別': ['総数'] * 5,
                            '表章項目': ['都道府県（自都市）内移動者数'] * 5},
                           index=pd.DatetimeIndex(['2009-03-01', '2009-02-01', '2009-01-01',
                                                   '2008-12-01', '2008-11-01'], name='時間軸（月次）'))
        tm.assert_frame_equal(df.head(), exp)

    def test_data_estat_data_numeric(self):
        ESTAT_KEY = os.environ['ESTAT']
        df = jpd.DataReader('0003109612', 'estat', appid=ESTAT_KEY)
        self.assertEqual(df['value'].dtype, np.float64)


if __name__ == '__main__':
    import nose
    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
                   exit=False)



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

import xml.etree.ElementTree as ET

import numpy as np
import pandas as pd

from pandas_datareader.base import _BaseReader
from japandas.tseries.tools import to_datetime


# http://www.e-stat.go.jp/api/e-stat-manual/

METADATA_MAPPER = {
    # 'TABLE_INF': '統計表ID',
    'STAT_NAME': '政府統計名',
    'GOV_ORG': '作成機関名',
    'STATISTICS_NAME': '提供統計名及び提供分類名',
    'TITLE': '統計表題名及び表番号',
    'CYCLE': '提供周期',
    'SURVEY_DATE': '調査年月',
    'OPEN_DATE': '公開日',
    'SMALL_AREA': '小地域属性フラグ',
    'MAIN_CATEGORY': '統計大分野名',
    'SUB_CATEGORY': '統計小分野名',
    'OVERALL_TOTAL_NUMBER': '総件数',
    'UPDATED_DATE': '最終更新日',
    'id': '統計表ID'
}


class EStatReader(_BaseReader):

    def __init__(self, symbols=None, appid=None, **kwargs):
        if isinstance(symbols, pd.DataFrame):
            if '統計表ID' in symbols.columns:
                symbols = symbols.loc[:, '統計表ID']
            else:
                raise ValueError('DataFrame 中に "統計表ID" カラムがありません')

        super(EStatReader, self).__init__(symbols=symbols, **kwargs)

        if appid is None:
            raise ValueError('アプリケーションID "appid" を文字列で指定してください')
        self.appid = appid

    @property
    def url(self):
        return 'http://api.e-stat.go.jp/rest/2.0/app/getStatsData'

    @property
    def params(self):
        return {'appId': self.appid, 'lang': 'J'}

    def read(self):
        """ read data """
        if isinstance(self.symbols, pd.compat.string_types):
            if len(self.symbols) == 8:
                return self.get_estat_list()

            params = self.params
            params['statsDataId'] = self.symbols
            return self._read_one_data(self.url, params)

        elif pd.core.common.is_list_like(self.symbols):
            dfs = []
            for symbol in self.symbols:
                params = self.params
                params['statsDataId'] = symbol
                df = self._read_one_data(self.url, params)
                dfs.append(df)

            if len(dfs) == 0:
                raise ValueError('取得するIDがありません')
            elif len(dfs) == 1:
                return dfs[0]
            else:
                return dfs[0].append(dfs[1:])
        else:
            raise ValueError('IDは文字列もしくはそのリストで指定してください')

    def _read_lines(self, out):
        root = ET.fromstring(out.getvalue())
        # retrieve class
        class_names = {}   # mapping from class id to name
        class_codes = {}   # mapping from class id to codes
        for c in root.findall('.//CLASS_OBJ'):
            class_id = c.attrib['id']
            class_names[class_id] = c.attrib['name']

            mapper = {}
            for code in c.findall('CLASS'):
                mapper[code.attrib['code']] = code.attrib['name']
            class_codes[class_id] = mapper

        # retrieve values
        values = []
        for value in root.findall('.//VALUE'):
            row = {}
            for cat in class_codes:
                name = class_names[cat]
                code = value.attrib[cat]
                row[name] = class_codes[cat][code]

            if value.text in ('-', ):
                # avoid to_numeric fails
                row['value'] = np.nan
            else:
                row['value'] = value.text
            values.append(row)

        df = pd.DataFrame(values)
        df.loc[:, 'value'] = pd.to_numeric(df['value'], errors='ignore')

        if 'time' in class_names:
            df = df.set_index(class_names['time'])
            df.index = to_datetime(df.index)
        return df

    def get_estat_list(self):
        url = 'http://api.e-stat.go.jp/rest/2.0/app/getStatsList'
        params = {'appId': self.appid, 'lang': 'J', 'statsCode': self.symbols}

        out = self._read_url_as_StringIO(url, params=params)
        root = ET.fromstring(out.getvalue())

        values = []
        columns = []
        for table in root.findall('.//TABLE_INF'):
            columns = ['統計表ID']
            row = {'統計表ID': table.get('id')}
            for elem in table.iter():
                if elem.tag == 'TABLE_INF':
                    continue

                if elem.tag in ('UPDATED_DATE', 'OPEN_DATE'):
                    val = pd.to_datetime(elem.text)
                elif elem.tag == 'SURVEY_DATE':
                    # Almost impossible to parse SURVEY_DATE as Timestamp...
                    val = elem.text
                elif elem.tag == 'OVERALL_TOTAL_NUMVER':
                    val = pd.to_numeric(elem.text)
                else:
                    val = elem.text
                label = METADATA_MAPPER.get(elem.tag, elem.tag)
                columns.append(label)
                row[label] = val
            values.append(row)

        if len(values) == 0:

            try:
                # if msg can be extracted from XML, raise it
                root = ET.fromstring(out.getvalue())
                msg = root.find('RESULT').find('ERROR_MSG').text
            except Exception:
                # otherwie, raise all XML content
                raise ValueError(out.getvalue())
            raise ValueError(msg.encode('utf-8', 'replace'))

        df = pd.DataFrame(values, columns=columns)
        return df



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

from japandas.io.estat import EStatReader

from pandas_datareader import data


_ohlc_columns_jp = ['始値', '高値', '安値', '終値', '出来高', '調整後終値*']
_ohlc_columns_en = ['Open', 'High', 'Low', 'Close', 'Volume', 'Adj Close']


def DataReader(symbols, data_source=None, start=None, end=None, appid=None, **kwargs):
    if data_source == 'yahoojp':
        msg = "YahooJPReaderは削除されました https://www.yahoo-help.jp/app/answers/detail/p/546/a_id/93575"
        raise NotImplementedError(msg)
    elif data_source == 'estat':
        return EStatReader(symbols=symbols, appid=appid, **kwargs).read()
    else:
        return data.DataReader(name=symbols, data_source=data_source,
                               start=start, end=end, **kwargs)


DataReader.__doc__ = data.DataReader.__doc__



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals

import numpy as np
import pandas as pd
import pandas.compat as compat
import pandas.core.common as com


_formats = ['%Y年', '%Y年%m月%d日', '%Y年%m月',
            '%Y年%m月%d日%H時%M分', '%Y年%m月%d日%H時%M分%S秒',

            '%y年', '%y年%m月%d日', '%y年%m月',
            '%y年%m月%d日%H時%M分', '%y年%m月%d日%H時%M分%S秒',

            '%m月%d日', '%m月%d日%H時%M分', '%m月%d日%H時%M分%S秒']


def to_datetime(arg, box=True, format=None, **kwargs):

    try:
        result = pd.to_datetime(arg, box=box, format=format, **kwargs)

        if format is not None:
            # if format is specified, return pd.to_datetime as it is
            return result

        if result is None:
            return result
        elif isinstance(result, (pd.Timestamp, pd.DatetimeIndex)):
            return result
    except ValueError:
        # as of pandas 0.17, to_datetime raises when parsing fails
        result = arg

    def _convert_listlike(arg, box):
        for format in _formats:
            try:
                return pd.to_datetime(arg, box=box, format=format, **kwargs)
            except ValueError:
                pass
        return arg

    if isinstance(result, compat.string_types):
        arg = np.array([arg], dtype='O')
        result = _convert_listlike(arg, box)
        return result[0]

    if isinstance(result, pd.Series):
        values = _convert_listlike(arg.values, False)
        return pd.Series(values, index=arg.index, name=arg.name)
    elif com.is_list_like(result):
        return _convert_listlike(result, box)
    return result


def date_range(start=None, end=None, **kwargs):
    start = to_datetime(start)
    end = to_datetime(end)
    return pd.date_range(start=start, end=end, **kwargs)


def period_range(start=None, end=None, **kwargs):
    start = to_datetime(start)
    end = to_datetime(end)
    return pd.period_range(start=start, end=end, **kwargs)


to_datetime.__doc__ = pd.to_datetime.__doc__
date_range.__doc__ = pd.date_range.__doc__
period_range.__doc__ = pd.period_range.__doc__


"""
try:
    import pandas.tseries.timedeltas as timedeltas
    abbrevs = [('d' ,'days|d|day|日'),
               ('h' ,'hours|h|hour|時間'),
               ('m' ,'minutes|min|minute|m|分'),
               ('s' ,'seconds|sec|second|s|秒'),
               ('ms','milliseconds|milli|millis|millisecond|ms'),
               ('us','microseconds|micro|micros|microsecond|us'),
               ('ns','nanoseconds|nano|nanos|nanosecond|ns')]
    timedeltas.abbrevs = abbrevs
except Exception:
    pass
"""
